 /*---------------------------------------------------------------------------
                  Copyright (c) 2005 - 2007 demmel products
         This example demo code is provided as is and has no warranty,
     implied or otherwise.  You are free to use/modify any of the provided
    code at your own risk in your applications with the expressed limitation
    of liability (see below) so long as your product using the code contains
                      at least one uPSD products (device).

    LIMITATION OF LIABILITY:   NEITHER demmel products NOR ITS VENDORS OR
   AGENTS SHALL BE LIABLE FOR ANY LOSS OF PROFITS, LOSS OF USE, LOSS OF DATA,
       INTERRUPTION OF BUSINESS, NOR FOR INDIRECT, SPECIAL, INCIDENTAL OR
       CONSEQUENTIAL DAMAGES OF ANY KIND WHETHER UNDER THIS AGREEMENT OR
         OTHERWISE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
  --------------------------------------------------------------------------*/

 /*---------------------------------------------------------------------------

	Version 1.1

   This simple example program shows how to use the I2C mode of the iLCD
   controller. It compiles on Keil-C for 8051, but it can be easily
   transferred to a different compiler and/or micro-controller.

	IMPORTANT NOTE: Ensure your iLCD controller fully supports I2C
	DPC10xx greater version 2.04, DPC20xx and DPC30xx greater or equal
   version 1.13
  --------------------------------------------------------------------------*/

#include "i2c.h"
#include "hardware.h"
#include <string.h>			// for strlen()

// Command bytes sent to/received from iLCD controller
#define CIN_CHAR		0xAA
#define EOT_CHAR		'\0'
#define ACK_CHAR		'\x06'
#define NACK_CHAR		'\x15'
#define ACK_SEQ		"\x06"
#define NACK_SEQ		"\x15"
#define CIN_SEQ		"\xAA"

//----------------------------------------------------------------------------
bool send_i2c_data(byte *string, byte len)
{
	word retries = 1000;
	byte in_byte;

	if (!i2c_write_block(string, len))
	{
		// Write failed
		return false;
	}

	// To make this example program more simple, we do not check any timeout here
	// but we simply retry getting a response up to 1000 times. This approach is very
	// resource consuming (and even consumes a lot of processing resources in the
	// iLCD controller itself) and SHOULD BE NEVER USED IN A REAL DESIGN!

	// A more elegant and resource-saving approach would be to monitor the ALERT pin
	// (preferred via an interrupt) and to retrieve data from the iLCD controller only
	// if the ALERT pin tells us that data is available.
	// If it is not possible to use the ALERT pin, a delay of about 1 to 10 ms
	// between 2 consecutive retries should be inserted.

	while (retries--)
	{
		in_byte = i2c_read_data_size();

		if (!i2c_get_result())
		{
			// issuing a command immediately after i2c_write_block() may fail when
			// the iLCD controller is not ready to receive a command yet.
			// We do not signal an error in this case, but retry another time.
			continue;
		}

		if (in_byte)
		{
			if ((i2c_read_block(&in_byte, 1) & 0xff) != 1)
			{
				// i2c_read_block() failed
				return false;
			}

			if (in_byte == ACK_CHAR)
			{
				// Bingo!
				return true;
			}
			else
			{
				// got something else than an ACK
				return false;
			}
		}
	}

	// # of retries expired
	return false;
}
//----------------------------------------------------------------------------
// assuming 'data' is 0-terminated
bool send_command(byte *string)
{
	byte count = strlen(string);

	return send_i2c_data(string, count);
}
//----------------------------------------------------------------------------
bool init_ilcd()
{
	// Init & erase screen
	return send_command(CIN_SEQ "!");
}
//----------------------------------------------------------------------------
bool send_text(byte *text)
{
	// Send command start "manually"
	if (!i2c_write_block(CIN_SEQ "DT", 3))
		return false;

	// Note that we have to transmit the terminating 0-byte as well
	return send_i2c_data(text, strlen(text) + 1);
}
//----------------------------------------------------------------------------
void main()
{
	bool retval;
	void *dummy;
	void *dummy1;

	// We do not use function i2c_read_status() and i2c_read_info() in this example program.
	// To avoid complaining of the linker (ignored for overlay process)
	// assign it to a dummy variable.
	dummy = i2c_read_status;
	dummy1 = i2c_read_info;

	// Initialize I2C stuff
	i2c_init(I2C_ADDRESS);

	// do not forget to switch off or service any watchdog which might be enabled
	// by default on your specific controller!

	retval = init_ilcd();

	if (retval)
		retval = send_text("I2C Test Program\r");

	// we have a micro-controller program - don't fall out of main()
	while (true)
		;
}
//----------------------------------------------------------------------------
